#include <gtk/gtk.h>

#include "boilerplate-application.h"
#include "boilerplate-window.h"

G_DEFINE_TYPE_WITH_PRIVATE (BoilerplateApplication, boilerplate_application, GTK_TYPE_APPLICATION);

const GOptionEntry option_entries[] = {
    { "debug", 'd', 0, G_OPTION_ARG_NONE, NULL, "Run in debug mode and show extra output", NULL },
    { "version", 'v', 0, G_OPTION_ARG_NONE, NULL, "Print release version and exit", NULL },
    { "disable-csd", 'c', 0, G_OPTION_ARG_NONE, NULL, "Disable Client Side Decorations (CSD)", NULL },
    { NULL }
};

static GActionEntry action_entries[] = {
    { "preferences", boilerplate_application_action_preferences, NULL, NULL, NULL },
    { "shortcuts", boilerplate_application_action_shortcuts, NULL, NULL, NULL },
    { "help", boilerplate_application_action_help, NULL, NULL, NULL },
	{ "about", boilerplate_application_action_about, NULL, NULL, NULL },
	{ "quit", boilerplate_application_action_quit,  NULL, NULL, NULL },
};

static void
boilerplate_application_action_preferences (GSimpleAction *action,
                                            GVariant      *parameter,
                                            gpointer      app) {
    
}

static void
boilerplate_application_action_shortcuts (GSimpleAction *action,
                                          GVariant      *parameter,
                                          gpointer      app) {
    
}

static void
boilerplate_application_action_help (GSimpleAction *action,
                                     GVariant      *parameter,
                                     gpointer      app) {
    
}

static void
boilerplate_application_action_about (GSimpleAction *action,
                                      GVariant      *parameter,
                                      gpointer      app) {
	static const char *authors[] = {
		"Julian Richen <julian@richen.io>",
		NULL,
	};

	static const char *artists[] = {
		"Julian Richen <julian@richen.io>",
		NULL,
	};

	static const char *translators[] = {
		"Julian Richen <julian@richen.io>",
		NULL,
	};

	gtk_show_about_dialog (gtk_application_get_active_window (GTK_APPLICATION (app)),
			       "program-name", g_get_application_name (),
			       "version", PACKAGE_VERSION,
			       "copyright", "Copyright \xc2\xa9 "COPYRIGHT" Julian Richen",
			       "comments", _("Short description of application."),
			       "authors", authors,
			       "artists", artists,
			       "translator-credits", translators,
			       "website-label", _("Website"),
			       "website", PACKAGE_URL,
			       "logo-icon-name", gtk_window_get_default_icon_name (),
			       "wrap-license", TRUE,
			       "license-type", GTK_LICENSE_GPL_3_0,
			       NULL);
}

static void
boilerplate_application_action_quit (GSimpleAction *action,
                                     GVariant      *parameter,
                                     gpointer      app) {
    g_application_quit (G_APPLICATION (app));
}

static void
boilerplate_application_init_accelerators (GtkApplication *application) {
	/* Taken from eog, which in turn has this based on a simular
	 * construct in Evince (src/ev-application.c).
	 * Setting multiple accelerators at once for an action
	 * is not very straight forward in a static way.
	 *
	 * This gchar* array simulates an action<->accels mapping.
	 * Enter the action name followed by the accelerator strings
	 * and terminate the entry with a NULL-string.*/
	static const gchar *const accelmap[] = {
        "app.quit", "<Primary>q", NULL,
		NULL
	};

	const gchar *const *it;
	for (it = accelmap; it[0]; it += g_strv_length ((gchar **)it) + 1) {
		gtk_application_set_accels_for_action (application, it[0], &it[1]);
	}
}

static void
boilerplate_application_startup (GApplication *application) {
	BoilerplateApplication *app;
    GtkBuilder *builder;
    GMenuModel *app_menu;
    
    app = BOILERPLATE_APPLICATION (application);

	G_APPLICATION_CLASS (boilerplate_application_parent_class)->startup (application);

	gtk_window_set_default_icon_name ("boilerplate");
	g_set_application_name (_("Boilerplate for C & GTK3"));

	g_action_map_add_action_entries (G_ACTION_MAP (application),
                                     action_entries, G_N_ELEMENTS (action_entries),
                                     application);

	boilerplate_application_init_accelerators (GTK_APPLICATION (app));
    
    /*
    builder = gtk_builder_new_from_resource ("/io/github/julianrichen/boilerplate-for-c-gtk3/app-menu.ui");
    app_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "appmenu"));
    gtk_application_set_app_menu (GTK_APPLICATION (app), app_menu);
    g_object_unref (builder);
    */
}

static void
boilerplate_application_activate (GApplication *application) {
	BoilerplateWindow *window;
    
	window = boilerplate_application_window_new (BOILERPLATE_APPLICATION (application));
	gtk_window_present (GTK_WINDOW (window));
}

static void
boilerplate_application_dispose (GObject *object) {
    /*
	BoilerplateApplicationPrivate *priv;

	priv = boilerplate_application_get_instance_private (BOILERPLATE_APPLICATION (object));

	g_clear_object (&priv->colors);
    */
	G_OBJECT_CLASS (boilerplate_application_parent_class)->dispose (object);
}

static void
boilerplate_application_init (BoilerplateApplication *application) {
	g_application_add_main_option_entries (G_APPLICATION (application), option_entries);
    
    /*
    application->settings = g_settings_new("io.github.julianrichen.boilerplate-for-c-gtk3");
    */
}

static void
boilerplate_application_class_init (BoilerplateApplicationClass *class) {
	G_APPLICATION_CLASS (class)->startup = boilerplate_application_startup;
	G_APPLICATION_CLASS (class)->activate = boilerplate_application_activate;
}

BoilerplateApplication *
boilerplate_application_new (void) {
    return g_object_new (BOILERPLATE_APPLICATION_TYPE,
                         "application-id", "io.github.julianrichen.boilerplate-for-c-gtk3",
                         "flags", G_APPLICATION_FLAGS_NONE,
                         NULL);
}